<?php

require_once 'base/GpealtesitufuncEntityBase.php';


/**
 * GpealtesitufuncEntity.
 *
 * @package Entity
 *
 * @author  alessandrobartels
 *
 */
class GpealtesitufuncEntity extends GpeAltesitufuncEntityBase
{

    const SIT_FUNC_NORMAL   = 1;
    const SIT_FUNC_SAL_MATERNIDADE = 5;/*BORBA 01/12/2014 Para atender a solicitação 153873*/
    const SIT_FUNC_DEMITIDO = 13;
    const SIT_FUNC_ATESTADO = 12; /*BORBA 19/02/2015 Para atender a solicitação 154336*/
    const SIT_FUNC_DOENCA_MENOR_15 = 15;/*BORBA 10/09/2015 Para atender a solicitação 154336*/

    private $_nrOrgTrabalho;
    private $_nrOrgPadrao;
    private $_mesCompetencia;


    /**
     * Construtor.
     *
     * @param integer $id            Identificador.
     * @param integer $nrOrgPadrao   Numero da Organizacao Padrao.
     * @param integer $nrOrgTrabalho Numero da Organizacao de Trabalho.
     *
     * @author alessandrobartels - 05/07/2011
     */
    public function __construct($id = null, $nrOrgPadrao = null, $nrOrgTrabalho = null, $mesCompetencia = null){
        parent::__construct($id);
        $this->_nrOrgPadrao = $nrOrgPadrao;
        $this->_nrOrgTrabalho = $nrOrgTrabalho;
        $this->_mesCompetencia = $mesCompetencia;
    }


    /**
     * Adiciona na lista os itens de alteracao funcional.
     *
     * @param integer $nr_org       Numero da Organizacao.
     * @param integer $nr_vinculo   Numero do Vinculo.
     * @param integer $dt_admissao  Data de Admissão.
     * @return void
     *
     * @author alessandrobartels
     */
    public function retornaAlteracaoSituacaoFuncional($nr_org, $nr_vinculo, $dt_admissao) {
        $this->resetItems();

        $this->setNrvinculom($nr_vinculo);
        $this->setNrorg($nr_org);

        $resultado = Gpealtesitufunc::retornaItem($nr_vinculo, $nr_org);
        if (count($resultado) === 0) {
            // alessandrobartels - 21/12/2012
            // Nao tendo nenhuma alteracao de SF para o vinculo, cria-se uma com SF normal na competencia
            $this->incluiAltSituFuncional(
                $nr_vinculo, $nr_org, GpesitufuncmEntity::SITUACAO_FUNCIONAL_NORMAL, $dt_admissao
            );
            $resultado = Gpealtesitufunc::retornaItem($nr_vinculo, $nr_org);

            //  tk_warning('Nao existe alteracao de situacao funcional para o vinculo '.$nr_vinculo.'.');
        } 

        foreach ($resultado as $entity_obj) {
            $this->addItem($entity_obj, $entity_obj->getNraltesitufunc());
        }
    }


    /**
     * Retorna o numero de meses de afastamento.
     *
     * @param TkDateTime $data_ini_per Data de inicio do periodo.
     * @param TkDateTime $data_fim_per Data de fim de periodo.
     *
     * @return integer
     *
     *@author alessandrobartels
     *
     * Metodo nao utilizado - VERIFICAR.
     */
    public function retornaMesesAfastamento(TkDateTime $data_ini_per, TkDateTime $data_fim_per) {

        $meses_afastados = 0;
        $data_ini_afastamento = new TkDateTime();
        $data_fim_afastamento = new TkDateTime();

        foreach ($this->getItems() as $item) {
            $data_ini_afastamento->set($item->getDtinisitufunc(), TkDateTime::FORMAT_DATE);

            if ($data_ini_afastamento->isEarlier($data_ini_per) === true) {
                $data_ini_afastamento->set($data_ini_per);
            }

            if ($item->getDtfimsitufunc() !== null) {
                $data_fim_afastamento->set($item->getDtfimsitufunc(), TkDateTime::FORMAT_DATE);

                if ($data_fim_afastamento->isLater($data_fim_per) === true) {
                    $data_fim_afastamento->set($data_fim_per);
                }
            } else {
                $data_fim_afastamento->set($data_fim_per, TkDateTime::FORMAT_DATE);
            }

            $meses_afastados += $data_ini_afastamento->monthsAfter($data_fim_afastamento);
        }

        return $meses_afastados;

    }


    /**
     * Retorna dias de afastamento.
     *
     * @param GpesitufuncmEntity $situacao_funcional Situacao Funcional.
     * @param TkDateTime         $data_ini_per       Data de inicio de periodo.
     * @param TkDateTime         $data_fim_per       Data de fim de periodo.
     * @param boolean            $determina_prox_sf  Proxima situacao funcional.
     *
     * @return array
     *
     * @author alessandrobartels
     *         brunofeliciano - 01/09/2011 - correcao de complexidade ciclomatica.
     */
    public function retornaDiasAfastamento(
        GpesitufuncmEntity $situacao_funcional = null, TkDateTime $data_ini_per, TkDateTime $data_fim_per,
        $determina_prox_sf
    ) {
        // rigel 23/02/2013 - alterados todos os for de 0 com <= para <
        $afastamento = array();

        //Quando a $data_ini_per e a $data_fim_per forem iguais o daysafter retorna 0       BORBA 10/07/2013
        //Utilizado também no método retornaDiasAfastamento na classe GpesitufuncmEntity para gerar afastamento
        //e no metodo carregaDiasUteis dentro de CalendarioLogic
        if($data_ini_per==$data_fim_per){
            $dias_periodo = 1;
        } else {
            $dias_periodo = $data_ini_per->daysAfter($data_fim_per);
        }

        /*o array é preenchido com -1 em todas as posições e onde a situação for normal ele preenche na
        posição o valor 1, onde houve afastamento é preenchido o número do afastamento para que seja gerado o evento
        correspondente  */
        for ($i=0; $i < $dias_periodo; $i++) {
            $afastamento[$i] = -1;
        }

        $persist = false;
        $data_fim_flag = true;
        $data_ini_aux = new TkDateTime();
        $data_fim_aux = new TkDateTime();
        $cont_alt_sits = (count($this->getItems()));
        $cont_itera = 0;
        foreach ($this->getItems() as $item) {
            $cont_itera++;
            $this->_verificaProxSituacaoFuncional(
                $determina_prox_sf, $situacao_funcional, $item, $data_fim_per, $persist
            );

            $data_ini_aux->set($item->getDtinisitufunc(), TkDateTime::FORMAT_DATE);

            if ($data_ini_aux->isLater($data_fim_per) === true) {
                // Afastamento comecou depois do periodo tratado.
                break;
            }

            /* @todo Verificar o tipo de retorno do item */

            $data_fim_aux = $this->_retornaDataFimSituacao($item->getDtfimsitufunc(), $data_fim_per, $data_fim_aux,$cont_alt_sits,$cont_itera);

            // Dentro do periodo aquisitivo.
            if ((($data_ini_aux->isLater($data_ini_per) === true)
                ||($data_ini_aux->equals($data_ini_per) === true))
                &&(($data_ini_aux->isEarlier($data_fim_per) === true)
                ||($data_ini_aux->equals($data_fim_per) === true))
            ) {
                /* Comecou e acabou durante.
                 * Conta dias de data_ini_aux ate data_fim_aux.
                 */
                $dia_ini = $data_ini_per->daysAfter($data_ini_aux) - 1;

                //Quando a data_ini_aux e a data_fim_aux forem iguais o daysafter retorna 0       BORBA 19/04/2013
                //Utilizado também no método retornaDiasAfastamento na classe GpesitufuncmEntity para gerar afastamento
                //e no metodo carregaDiasUteis dentro de CalendarioLogicpara geras dias periodo
                if($data_ini_aux==$data_fim_aux){
                    $qtd_dias = 1;
                } else {
                    $qtd_dias = $data_ini_aux->daysAfter($data_fim_aux);
                }
                // Ajustado para 0 quando negativo pois o array começa de 0 e nos demais casos trabalha com a posicao anterior;
                if ($dia_ini < 0){
                    $dia_ini = 0;
                }

                for ($i=0; $i < $qtd_dias; $i++) {
                    $afastamento[$i+$dia_ini] = (int)$item->getNrsitufuncm();
                }
            } elseif (($data_ini_aux->isEarlier($data_ini_per) === true)
                    &&(($data_fim_aux->isEarlier($data_fim_per) === true)
                    || ($data_fim_aux->equals($data_fim_per) === true))
                    || ($data_fim_aux->equals($data_ini_per) === true)
            ) {
                if (($data_fim_aux->isLater($data_ini_per) === true)
                    ||($data_fim_aux->equals($data_ini_per) === true)
                ) {
                    /* Comecou antes e acabou durante.
                     * Conta dias do data_ini_per ate data_fim_aux.
                     */

                    //Quando a data_ini_aux e a data_fim_aux forem iguais o daysafter retorna 0       BORBA 19/04/2013
                    //Utilizado também no método retornaDiasAfastamento na classe GpesitufuncmEntity para gerar afastamento
                    //e no metodo carregaDiasUteis dentro de CalendarioLogicpara geras dias periodo
                    if($data_ini_per==$data_fim_aux){
                        $qtd_dias = 1;
                    } else {
                        $qtd_dias = $data_ini_per->daysAfter($data_fim_aux);
                    }                    

                    for ($i=0; $i < $qtd_dias; $i++) {
                        $afastamento[$i] = (int)$item->getNrsitufuncm();
                    }
                } /*else {
                    // Comecou e acabou antes.
                }*/
            }
        }

        if ($persist === true) {
            $this->persist();
        }

        return $afastamento;
    }

    /*
    private function _calculoAfastamento(
        TkDateTime $data_ini_aux, TkDateTime $data_fim_aux, TkDateTime $data_ini_per, TkDateTime $data_fim_per,
        array $afastamento
    ){

        return $afastamento;
    }
    */


    /**
     * Determina a proxima situacao funcional.
     *
     * @param GpesitufuncmEntity    $situacao_funcional Situacao Funcional.
     * @param GpealtesitufuncEntity $item               Situacao Funcional.
     * @param TkDateTime            $data_fim_per       Data de fim de periodo.
     *
     * @return boolean
     *
     * @author alessandrobartels
     */
    private function _determinaProximaSituacaoFuncional(
        GpesitufuncmEntity $situacao_funcional, GpealtesitufuncEntity $item, TkDateTime $data_fim_per
    ) {
        $data_inicio_afastamento = new TkDateTime($item->getDtinisitufunc(), TkDateTime::FORMAT_DATE);
        $data_fim_afastamento = $item->getDtfimsitufunc();
        $persist = false;
        if (empty($data_fim_afastamento) === true) {
            // Projeta data fim.
            $sit_func = $situacao_funcional->retornaSituacaoFuncional(
                $this->_nrOrgPadrao, $this->_nrOrgTrabalho, $item->getNrsitufuncm()
            );

            $qtd_limite_afastamento = (int)$sit_func->historico->getQtdiaslimitsitu();

            /* @todo testar se $qtd_limite_afastamento <= 0 (empty) */
            if (empty($qtd_limite_afastamento) === true
                ||($qtd_limite_afastamento === 0)
            ) {
                // Se nao existe limite na ha como determinar proxima situacao.
                return $persist;
            }

            $data_fim_afastamento = new TkDateTime($data_inicio_afastamento, TkDateTime::FORMAT_DATE);
            $data_fim_afastamento->addDay($qtd_limite_afastamento - 1);
            if ($data_fim_afastamento->isEarlier($data_fim_per) === true) {
                // Encerra afastamento.
                $item->setDtfimsitufunc($data_fim_afastamento);
                $persist = true;
            }

            /* Uma proxima situacao no registro de alteracao (Gpealtesitufunc) tem prioridade sobre
             * uma proxima situacao na definicao da situacao funcional (Gpesitufunc).
             */
            $prox_situacao_func = $item->getNrproxsitufunc();
            if ($prox_situacao_func === null) {
                $prox_situacao_func = $sit_func->historico->getNrproxsitufunc();
            }

            if ($prox_situacao_func !== null) {
                $data_ini_afastamento_aux = new TkDateTime($data_fim_afastamento, TkDateTime::FORMAT_DATE);

                $novo_item = new self();
                $novo_item->setNraltesitufunc(rand(1, 10000));
                $novo_item->setNrorg($this->getNrorg());
                /*
                 * $item->setNrmotivoafasta();
                 * $item->setNrproxsitufunc();
                 * $item->setCdtabecdi();
                 * $item->setCddiagnost();
                 */
                $novo_item->setNrsitufuncm($prox_situacao_func);
                $novo_item->setNrvinculom($this->getNrvinculom());
                $novo_item->setDtinisitufunc($data_ini_afastamento_aux->addDay(1));
                /*
                 *           $item->setDtfimsitufunc();
                 *			$item->setDtverifsitufunc();
                 * 			$item->setDtpreviretositu();
                 *			$item->setDtprorrsitufunc();
                 */
                $novo_item->setDtinclusao(now());
                $novo_item->setDtultatu(now());
                $novo_item->setNrorginclusao($this->getNrorg());
                /*			$item->setCdoperinclusao(); */
                $novo_item->setNrorgultatu($this->getNrorg());
                /*			$item->setCdoperultatu(); */

                $this->addItem($novo_item, $novo_item->getNraltesitufunc());

                $persist = true;
            }

            return $persist;
        }

    }

    //}


    /**
     * Inclui Alteracao Situacao Funcional.
     *
     * @param integer    $nrvinculom    Numero do Vinculo.
     * @param integer    $nrorg         Numero da Organizacao.
     * @param integer    $nrsitufuncm   Numero da Situacao Funcional.
     * @param TkDateTime $dtinisitufunc Data do Inicio da Situacao Funcional.
     *
     * @return boolean
     *
     * @author brunofeliciano - 30/08/2011
     */
    public function incluiAltSituFuncional($nrvinculom, $nrorg, $nrsitufuncm, TkDateTime $dtinisitufunc) {

        try{
            $altSituFuncional = new self();

            $altSituFuncional->setNrvinculom($nrvinculom);
            $altSituFuncional->setNrorg($nrorg);
            $altSituFuncional->setNrsitufuncm($nrsitufuncm);
            $altSituFuncional->setDtinisitufunc($dtinisitufunc);

            // $altSituFuncional->addItem($altSituFuncional, $altSituFuncional->getNraltesitufunc());.
            $altSituFuncional->persist();

            unset($altSituFuncional);

            return true;
        }catch (Exception $e){
            throw $e;
        }
    }


    /**
     * Retorna Alteracoes de Situacao Funcional de um determinado vinculo.
     *
     * @param integer $nrvinculo Numero do Vinculo.
     * @param integer $nrorg     Numero da Organizacao.
     *
     * @return array
     *
     * @author brunofeliciano - 30/08/2011
     */
    public function retornaAlteSituFuncporVinculo($nrvinculo, $nrorg) {
        return Gpealtesitufunc::retornaItem($nrvinculo, $nrorg);
    }

    /**
     * Retorna Alteracoes de Situacao Funcional de um determinado vinculo para um determinado CID.
     *
     * @param integer $nrvinculo  Numero do Vinculo.
     * @param integer $nrsituacao Numero da Situacao Funcional.
     * @param integer $nrcid      Numero do CID.
     * @param integer $nrorg      Numero da Organizacao.
     *
     * @return Gpealtesitufunc
     *
     * @author brunoborba 10/02/2015 Para atender a solicitação 154336
     */
    public function carregaAlteSitFuncporVinculoNrcid($nrvinculo,$nrsituacao,$nrcid,$nrorg) {
        $this->resetItems();
        $this->setNrvinculom($nrvinculo);
        $this->setNrorg($nrorg);

        $resultado = Gpealtesitufunc::retornaAlteSitFuncporVinculoNrcid($nrvinculo,$nrsituacao,$nrcid,$nrorg);

        foreach ($resultado as $entity_obj) {
            $this->addItem($entity_obj, $entity_obj->getNraltesitufunc());
        }
    }

    /**
     * Retorna Alteracoes de Situacao Funcional de um determinado vinculo para uma determinada situação.
     *
     * @param integer $nrvinculo  Numero do Vinculo.
     * @param integer $nrsituacao Numero da Situacao Funcional.
     * @param integer $nrorg      Numero da Organizacao.
     *
     * @return Gpealtesitufunc
     *
     * @author brunoborba 27/11/2014 Para atender a solicitação 153873
     */
    public function retornaAlteSitFuncporVinculoNrsituacao($nrvinculo,$nrsituacao,$nrorg) {
        return Gpealtesitufunc::retornaAlteSitFuncporVinculoNrsituacao($nrvinculo,$nrsituacao,$nrorg);
    }

    /**
     * Retorna a Data Fim da Situacao.
     *
     * @param string     $data_fim     Data fim.
     * @param TkDateTime $data_fim_per Data fim do periodo.
     * @param TkDateTime $data_fim_aux Data fim auxiliar.
     *
     * @return TkDateTime
     *
     * @author brunofeliciano/alessandrobartels - 02/09/2011
     */
    private function _retornaDataFimSituacao($data_fim, TkDateTime $data_fim_per, TkDateTime $data_fim_aux, $cont_alt_sits,$cont_itera) {
        $data_fim_flag = true;
        if ($data_fim != null) {
            $data_fim_aux->set($data_fim, TkDateTime::FORMAT_DATE);
        } else {
            $data_fim_flag = false;
        }

        if (($data_fim_flag === false)
            ||($data_fim_aux->isLater($data_fim_per) === true)
        ) {
            $data_fim_aux->set($data_fim_per);
        }

        return $data_fim_aux;
    }


    /**
     * Verifica Proxima Situacao Funcional.
     *
     * @param boolean               $determina_prox_sf  Flag de Proxima Situacao Funcional.
     * @param GpesitufuncmEntity    $situacao_funcional Situacao Funcional.
     * @param GpealtesitufuncEntity $item               Situacao Funcional.
     * @param TkDateTime            $data_fim_per       Data de fim de periodo.
     * @param boolean               &$persist           Flag para Persist.
     *
     * @return void
     *
     * @author brunofeliciano/alessandrobartels - 02/09/2011
     */
    private function _verificaProxSituacaoFuncional(
        $determina_prox_sf, GpesitufuncmEntity $situacao_funcional, GpealtesitufuncEntity $item,
        TkDateTime $data_fim_per, &$persist
    ) {
        if ($determina_prox_sf === true) {
            // So determina proxima situacao funcional se for calculo. Em caso de ferias nao determina.
            $proxSituacaoFuncional = $this->_determinaProximaSituacaoFuncional(
                $situacao_funcional, $item, $data_fim_per
            );
            if ($proxSituacaoFuncional === true) {
                $persist = true;
            }
        }
    }

    public function incluiAltSituFuncionalHistorico($nrvinculom, $nrorg, $nrsitufuncm, $registro) {

        try{

//            $dataIni = str_replace("/","",substr($registro["INICIO_AFASTAMENTO"],0,10));
//            $dataFim = str_replace("/","",substr($registro["FIM_AFASTAMENTO"],0,10));
            if($registro["INICIO_AFASTAMENTO"] == "" || $registro["INICIO_AFASTAMENTO"] == null){
                $dataIni = null;
            }else{
                $dataIni = $registro["INICIO_AFASTAMENTO"]." 00:00:00";
            }
            if($registro["FIM_AFASTAMENTO"] == "" || $registro["FIM_AFASTAMENTO"] == null){
                $dataFim = null;
            }else{
                $dataFim = $registro["FIM_AFASTAMENTO"]." 00:00:00";
            }

            $tipoAfastLetra = substr($registro["TIPO_AFASTAMENTO"],0,1);

            /////////////////////////// CONFERIR COM A ELIANE DA NUTRI QUAIS SÃO OS VALORES PARA AS LETRAS
            switch($tipoAfastLetra){
                case 'E':
                    $tipoAfast = 5;
                    break;
                case 'P':
                    $tipoAfast = 6;
                    break;
                case 'T':
                    $tipoAfast = 3;
                    break;
                case 'R':
                    $tipoAfast = 1; //////// NAO É UM.... MUDAR!!!!!!!!!!!!!
                    break;
                case 'I':
                    $tipoAfast = 1; //////// NAO É UM.... MUDAR!!!!!!!!!!!!!
                    break;
                default:
                    $tipoAfast = 1;

            }

            $altSituFuncional = new self();

            $altSituFuncional->setNrvinculom($nrvinculom);
            $altSituFuncional->setNrorg($nrorg);
            $altSituFuncional->setNrsitufuncm($nrsitufuncm);
            $altSituFuncional->setDtinisitufunc($dataIni);
            $altSituFuncional->setDtfimsitufunc($dataFim);
            $altSituFuncional->setNrsitufuncm($tipoAfast);
            //$altSituFuncional->setNrmotivoafasta($registro["MOTIVO_AFASTAMENTO"]);
            // $altSituFuncional->addItem($altSituFuncional, $altSituFuncional->getNraltesitufunc());.
            $altSituFuncional->persist();

            unset($altSituFuncional);

            return true;
        }catch (Exception $e){
            //die("SALVOU--".$dataIni);
            throw $e;
        }
    }

    public function setAlteracaoSituacaoFuncionario($nrorg, $nrvinculom, $nrsitufuncm, $nrmotivoafasta, $dtinisitufunc,
                                                    $dtfimsitufunc)
    {
        $this->setNrorg($nrorg);
        $this->setNrvinculom($nrvinculom);
        $this->setDtinisitufunc($dtinisitufunc);
        $this->setDtfimsitufunc($dtfimsitufunc);
        $this->setNrsitufuncm($nrsitufuncm);
        $this->setNrmotivoafasta($nrmotivoafasta);
    }

    /**
     * @return \GpesitufuncmEntity
     */
    public function getSituacaoFuncional() {
        $situfuncm = \Gpesitufuncm::retornaSituFuncPelaChave($this->getNrsitufuncm(), $this->getNrorg());
        return $situfuncm;
    }

}

